/* pconn: signals.c - Sets up signal handlers */
/* Mark Kamichoff <prox@penguinhosting.net>   */

#include <stdlib.h>
#include <stdio.h>
#include <netinet/in.h>
#include <signal.h>
#include <netdb.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <string.h>

#define SA struct sockaddr              /* Improves readability of code, eh */

void signalpipe(void);          // Handles a broken pipe
void sig_chld(void);            // Handles SIGCHLD
void signalinterrupt(void);
void setupsignals(void);        // Sets up signal handlers
void syntax();
void setupsignals();
int bytepass(struct hostent *, char *, char *);

void signalpipe(void)
{
	extern int listeningfd, clientfd, datafd;
        fprintf(stderr, "Debug: Caught a SIGPIPE, transfer interrupted\n");
}

void sig_chld(void)             // Only called if we are in LINUX
{
	extern int listeningfd, clientfd, datafd;
        pid_t pid;
        int stat;

        while((pid = waitpid(-1, &stat, WNOHANG)) > 0);

        return;
}

void signalinterrupt(void)      
{
	extern int listeningfd, clientfd, datafd;
        printf("Cleaning up and exiting gracefully...\n");
	close(datafd);
        close(clientfd);
        close(listeningfd); 
//        usleep(131072);
        exit(0);
}

void setupsignals(void)
{
	extern int listeningfd, clientfd, datafd;
#ifndef LINUX                           // We ignore SIGCHLD in Solaris
        signal(SIGCHLD,SIG_IGN);
#endif
#ifdef LINUX                            // We handle SIGCHLD in Linux
        signal(SIGCHLD,(void *) sig_chld);
#endif
        signal(SIGPIPE,(void *) signalpipe);
        signal(SIGINT,(void *) signalinterrupt);
        signal(SIGTERM,(void *) signalinterrupt);
}
